@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2009 Atsushi Konno All rights reserved.
+ * Copyright (C) 2005-2011 Atsushi Konno All rights reserved.
  * Copyright (C) 2005 QSDN,Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -21,6 +21,7 @@
 
 #include "mod_chxj.h"
 #include "chxj_cookie.h"
+#include "chxj_css.h"
 
 /*----------------------------------------------------------------------------*/
 /* Structure for CHTML4.0                                                     */
@@ -28,17 +29,29 @@
 typedef struct chtml50_t chtml50_t;
 
 struct chtml50_t {
-    Doc                 *doc;
-    char                *out;
-    int                 out_len;
-    int                 pre_flag;
-    int                 textarea_flag;
-    int                 font_flag;
+  Doc                 *doc;
+  char                *out;
+  int                 out_len;
+  int                 pre_flag;
+  int                 textarea_flag;
+  int                 font_flag;
 
-    device_table        *spec;
-    mod_chxj_config     *conf;
-    chxjconvrule_entry  *entryp;
-    cookie_t            *cookie;
+  device_table        *spec;
+  mod_chxj_config     *conf;
+  chxjconvrule_entry  *entryp;
+  cookie_t            *cookie;
+  css_stylesheet_t    *style;
+  css_prop_list_stack_t *css_prop_stack;
+
+  char                *pagetitle;
+};
+
+typedef struct _chtml50_flags_t chtml50_flags_t;
+struct _chtml50_flags_t {
+  int with_font_flag;
+  int with_blink_flag;
+  int with_div_flag;
+  int with_marquee_flag;
 };
 
 /*----------------------------------------------------------------------------*/
