@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2009 Atsushi Konno All rights reserved.
+ * Copyright (C) 2005-2011 Atsushi Konno All rights reserved.
  * Copyright (C) 2005 QSDN,Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -21,6 +21,7 @@
 
 #include "mod_chxj.h"
 #include "chxj_cookie.h"
+#include "chxj_css.h"
 
 /*----------------------------------------------------------------------------*/
 /* Structure for J-PHONE HTML                                                 */
@@ -28,24 +29,38 @@
 typedef struct jxhtml_t jxhtml_t;
 
 struct jxhtml_t {
-    Doc                 *doc;
-    char                *out;
-    int                 out_len;
-    int                 pre_flag;
-    int                 textarea_flag;
-    int                 h1_align_flag;
-    int                 h2_align_flag;
-    int                 h3_align_flag;
-    int                 h4_align_flag;
-    int                 h5_align_flag;
-    int                 h6_align_flag;
-    int                 font_flag;
-    int                 font_size_flag;
+  Doc                 *doc;
+  char                *out;
+  int                 out_len;
+  int                 pre_flag;
+  int                 textarea_flag;
+  int                 h1_align_flag;
+  int                 h2_align_flag;
+  int                 h3_align_flag;
+  int                 h4_align_flag;
+  int                 h5_align_flag;
+  int                 h6_align_flag;
+  int                 font_flag;
+  int                 font_size_flag;
+  int                 start_html_flag;
 
-    device_table        *spec;
-    mod_chxj_config     *conf;
-    chxjconvrule_entry  *entryp;
-    cookie_t            *cookie;
+  device_table        *spec;
+  mod_chxj_config     *conf;
+  chxjconvrule_entry  *entryp;
+  cookie_t            *cookie;
+  css_stylesheet_t    *style;
+  css_prop_list_stack_t *css_prop_stack;
+
+  char                *pagetitle;
+};
+
+typedef struct _jxhtml_flags_t jxhtml_flags_t;
+struct _jxhtml_flags_t {
+  int with_font_flag;
+  int with_blink_flag;
+  int font_color_flag;
+  int font_size_flag;
+  int with_div_flag;
 };
 
 /*----------------------------------------------------------------------------*/
