@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2009 Atsushi Konno All rights reserved.
+ * Copyright (C) 2005-2011 Atsushi Konno All rights reserved.
  * Copyright (C) 2005 QSDN,Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,14 +20,12 @@
 #include <string.h>
 #include <apr_pools.h>
 
-extern int chxj_chk_numeric(
-  const char*          s);
-
-extern int chxj_atoi(
-  const char*          s);
-
+extern int chxj_chk_numeric(const char *s);
+extern int chxj_atoi(const char *s);
+extern int chxj_axtoi(const char *s);
 extern int chxj_strcasenrcmp(apr_pool_t *p, const char *s1, const char *s2, int n);
 extern int chxj_starts_with(const char *str, const char *word);
+extern int chxj_strcount(const char *s, const char *str);
 extern char *chxj_add_slash_to_doublequote(apr_pool_t *pool, const char *str);
 #endif
 /*
