@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2008 Atsushi Konno All rights reserved.
+ * Copyright (C) 2005-2011 Atsushi Konno All rights reserved.
  * Copyright (C) 2005 QSDN,Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,25 +18,41 @@
 #define __CHXJ_XHTML_MOBILE_1_0_H__
 
 #include "mod_chxj.h"
+#include "chxj_css.h"
 
 /*----------------------------------------------------------------------------*/
-/* Structure for HDML                                                         */
+/* Structure for XHTML                                                        */
 /*----------------------------------------------------------------------------*/
 typedef struct xhtml_t xhtml_t;
 
 struct xhtml_t {
-    Doc                 *doc;
-    char                *out;
-    int                 out_len;
-    int                 pre_flag;
-    int                 textarea_flag;
-    int                 font_color_flag;
-    int                 font_size_flag;
+  Doc                 *doc;
+  char                *out;
+  int                 out_len;
+  int                 pre_flag;
+  int                 textarea_flag;
+  int                 font_color_flag;
+  int                 font_size_flag;
 
-    device_table        *spec;
-    mod_chxj_config     *conf;
-    chxjconvrule_entry  *entryp;
-    cookie_t            *cookie;
+  device_table        *spec;
+  mod_chxj_config     *conf;
+  chxjconvrule_entry  *entryp;
+  cookie_t            *cookie;
+  css_stylesheet_t    *style;
+  css_prop_list_stack_t *css_prop_stack;
+
+  int                 start_html_flag;
+
+  char                *pagetitle;
+};
+
+typedef struct _xhtml_flags_t xhtml_flags_t;
+struct _xhtml_flags_t {
+  int with_font_flag;
+  int with_blink_flag;
+  int font_color_flag;
+  int font_size_flag;
+  int with_div_flag;
 };
 
 /*----------------------------------------------------------------------------*/
@@ -51,4 +67,7 @@ extern char *chxj_convert_xhtml_mobile_1_0(
   chxjconvrule_entry    *entryp,
   cookie_t              *cookie);
 
+
+extern char *chxj_xhtml_emoji_only_converter(request_rec *r, device_table *spec, const char *src, apr_size_t len);
+
 #endif
