@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2009 Atsushi Konno All rights reserved.
+ * Copyright (C) 2005-2011 Atsushi Konno All rights reserved.
  * Copyright (C) 2005 QSDN,Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -189,6 +189,18 @@ struct jphone_emoji_t {
   char* string;
 };
 
+typedef struct iphone_emoji_t iphone_emoji_t;
+
+struct iphone_emoji_t {
+  char* string;
+};
+
+typedef struct android_emoji_t android_emoji_t;
+
+struct android_emoji_t {
+  char* string;
+};
+
 typedef struct emoji_t emoji_t;
 
 struct emoji_t {
@@ -197,6 +209,8 @@ struct emoji_t {
   imode_emoji_t*   imode;
   ezweb_emoji_t*   ezweb;
   jphone_emoji_t*  jphone;
+  iphone_emoji_t*  iphone;
+  android_emoji_t*  android;
 };
 
 typedef struct chxjconvrule_entry chxjconvrule_entry;
@@ -377,6 +391,15 @@ struct mod_chxj_config {
   int                   image_rewrite;
   char                  *image_rewrite_url;
   int                   image_rewrite_mode;
+
+  int                   use_emoji_image;
+  char                  *emoji_image_url;
+
+
+  int                   use_google_analytics;
+  char                  *google_analytics_account;
+  char                  *google_analytics_target;   /* i.e. /ga.pl */
+  char                  *google_analytics_debug;    /* debug filename */
 };
 
 struct mod_chxj_req_config_t {
@@ -523,6 +546,8 @@ extern tag_handler  xhtml_handler[];
 extern tag_handler  hdml_handler[];
 extern tag_handler  jhtml_handler[];
 extern tag_handler  jxhtml_handler[];
+extern tag_handler  iphone_handler[];
+extern tag_handler  android_handler[];
 
 extern char* chxj_node_convert( 
   device_table *spec,
@@ -550,6 +575,7 @@ extern void *rpl_malloc(size_t n);
 
 
 #define TO_ADDR(X) ((unsigned int)(apr_size_t)(X))
+#define ADJUST_WIDTH_FOR_ANDROID (40)
 #endif
 /*
  * vim:ts=2 et
