@@ -616,7 +616,6 @@ s_create_cache_file(request_rec          *r,
     }
   
     readdata = apr_palloc(r->pool, st->size);
-    readbyte = st->size;
     rv = apr_file_read_full(fin, (void*)readdata, st->size, &readbyte);
     apr_file_close(fin);
     if (rv != APR_SUCCESS || readbyte != st->size) {
@@ -667,8 +666,17 @@ s_create_cache_file(request_rec          *r,
     int oldw = MagickGetImageWidth(magick_wand);
     int oldh = MagickGetImageHeight(magick_wand);
     int done_fixup_size = 0;
+    int ww = spec->width;
+    int hh = spec->heigh;
+    if (IS_IPHONE(spec) || IS_ANDROID(spec)) {
+      ww = (int)((double)ww * (double)1.5);
+      hh = (int)((double)hh * (double)1.5);
+      if (IS_ANDROID(spec)) {
+        ww = ww - ADJUST_WIDTH_FOR_ANDROID;
+      }
+    }
     if ((qsp->mode == IMG_CONV_MODE_WALLPAPER && spec->wp_width < oldw && spec->wp_heigh < oldh)
-      || (qsp->mode != IMG_CONV_MODE_WALLPAPER && spec->width < oldw && spec->heigh < oldh)) {
+      || (qsp->mode != IMG_CONV_MODE_WALLPAPER && ww < oldw && hh < oldh)) {
       /*
        * The size of the image is changed.
        */
@@ -1107,8 +1115,17 @@ s_create_blob_data(request_rec          *r,
     int oldw = MagickGetImageWidth(magick_wand);
     int oldh = MagickGetImageHeight(magick_wand);
     int done_fixup_size = 0;
+    int ww = spec->width;
+    int hh = spec->heigh;
+    if (IS_IPHONE(spec) || IS_ANDROID(spec)) {
+      ww = (int)((double)ww * (double)1.5);
+      hh = (int)((double)hh * (double)1.5);
+      if (IS_ANDROID(spec)) {
+        ww = ww - ADJUST_WIDTH_FOR_ANDROID;
+      }
+    }
     if ((qsp->mode == IMG_CONV_MODE_WALLPAPER && spec->wp_width < oldw && spec->wp_heigh < oldh)
-      || (qsp->mode != IMG_CONV_MODE_WALLPAPER && spec->width < oldw && spec->heigh < oldh)) {
+      || (qsp->mode != IMG_CONV_MODE_WALLPAPER && ww < oldw && hh < oldh)) {
       /*
        * The size of the image is changed.
        */
@@ -1277,6 +1294,16 @@ s_fixup_size(MagickWand           *magick_wand,
   c_width = spec->width;
   c_heigh = spec->heigh;
 
+  if (IS_IPHONE(spec) || IS_ANDROID(spec)) {
+    c_width = (int)((double)c_width * (double)1.5);
+    c_heigh = (int)((double)c_heigh * (double)1.5);
+    if (IS_ANDROID(spec)) {
+      c_width = c_width - ADJUST_WIDTH_FOR_ANDROID;
+    }
+    DBG(r,"REQ[%X] detect iphone/android width=[%d]", TO_ADDR(r),c_width);
+    DBG(r,"REQ[%X] detect iphone/android heigh=[%d]", TO_ADDR(r),c_heigh);
+  }
+
   switch(mode) {
   case IMG_CONV_MODE_THUMBNAIL:
 
@@ -1810,28 +1837,68 @@ s_send_cache_file(
     }
     char *nowFormat = MagickGetImageFormat(magick_wand);
     DestroyMagickWand(magick_wand);
+    char *fname = apr_pstrdup(r->pool, r->filename);
+    char *ext = strrchr(fname, '.');
     if (nowFormat) {
       if (STRCASEEQ('j','J',"jpeg",nowFormat) || STRCASEEQ('j','J',"jpg",nowFormat)) {
         DBG(r,"REQ[%X] detect cache file => jpg.",TO_ADDR(r));
         ap_set_content_type(r, "image/jpeg");
+
+        if (!ext) {
+          fname = apr_pstrcat(r->pool, fname, ".jpg");
+        }
+        else if (ext && strcasecmp(".jpeg", ext) != 0 && strcasecmp(".jpg", ext) != 0) {
+          *ext = 0;
+          fname = apr_pstrcat(r->pool, fname, ".jpg");
+        }
       }
       else if (STRCASEEQ('p','P',"png", nowFormat)) {
         DBG(r,"REQ[%X] detect cache file => png.",TO_ADDR(r));
         ap_set_content_type(r, "image/png");
+
+        if (!ext) {
+          fname = apr_pstrcat(r->pool, fname, ".png");
+        }
+        else if (ext && strcasecmp(".png", ext) != 0) {
+          *ext = 0;
+          fname = apr_pstrcat(r->pool, fname, ".png");
+        }
+
       }
       else if (STRCASEEQ('g','G',"gif", nowFormat)) {
         DBG(r,"REQ[%X] detect cache file => gif.",TO_ADDR(r));
         ap_set_content_type(r, "image/gif");
+
+        if (!ext) {
+          fname = apr_pstrcat(r->pool, fname, ".gif");
+        }
+        else if (ext && strcasecmp(".gif", ext) != 0) {
+          *ext = 0;
+          fname = apr_pstrcat(r->pool, fname, ".gif");
+        }
       }
       else if (STRCASEEQ('b','B',"bmp", nowFormat)) {
         DBG(r,"REQ[%X] detect cache file => bmp.",TO_ADDR(r));
         ap_set_content_type(r, "image/bmp");
+
+        if (!ext) {
+          fname = apr_pstrcat(r->pool, fname, ".bmp");
+        }
+        else if (ext && strcasecmp(".bmp", ext) != 0) {
+          *ext = 0;
+          fname = apr_pstrcat(r->pool, fname, ".bmp");
+        }
       }
       else {
         ERR(r,"REQ[%X] detect unknown file",TO_ADDR(r));
         return HTTP_NOT_FOUND;
       }
     }
+    if (query_string->mode == IMG_CONV_MODE_WALLPAPER && (IS_IPHONE(spec)||IS_ANDROID(spec))) {
+      ap_set_content_type(r, "application/octet-stream");
+      apr_table_setn(r->headers_out, "Content-Disposition", apr_psprintf(r->pool, "attachment; filename=%s", fname));
+      DBG(r,"REQ[%X] detect cache file => iphone/android",TO_ADDR(r));
+    }
     if (conf->image_copyright) {
       DBG(r,"REQ[%X] Add COPYRIGHT Header for SoftBank [%s]", TO_ADDR(r), conf->image_copyright);
       if (spec->html_spec_type == CHXJ_SPEC_Jhtml ||  spec->html_spec_type == CHXJ_SPEC_Jxhtml) {
